@@ -169,6 +169,7 @@ class SASconfig(object):
         cfg = getattr(SAScfg, cfgname)
 
         ip            = cfg.get('ip')
+        url           = cfg.get('url')
         ssh           = cfg.get('ssh')
         path          = cfg.get('saspath')
         java          = cfg.get('java')
@@ -203,7 +204,7 @@ class SASconfig(object):
 
         inautoexec = kwargs.get('autoexec', None)
         if inautoexec:
-            if lock and self.autoexec:
+            if lock and self.autoexec is not None:
                 print("Parameter 'autoexec' passed to SAS_session was ignored due to configuration restriction.")
             else:
                 self.autoexec = inautoexec
@@ -212,43 +213,52 @@ class SASconfig(object):
         if inm5dsbug is not None:
            self.m5dsbug = inm5dsbug
 
+        inurl = kwargs.get('url', None)             
+        if inurl:
+           if lock and url is not None:
+              print("Parameter 'url' passed to SAS_session was ignored due to configuration restriction.")
+           else:
+              url = inurl   
+
         inip = kwargs.get('ip', None)             
         if inip:
-           if lock and len(ip):
+           if lock and ip is not None:
               print("Parameter 'ip' passed to SAS_session was ignored due to configuration restriction.")
            else:
               ip = inip   
 
         inssh = kwargs.get('ssh', None)
         if inssh:
-           if lock and len(ssh):
+           if lock and ssh is not None:
               print("Parameter 'ssh' passed to SAS_session was ignored due to configuration restriction.")
            else:
               ssh = inssh
 
         insaspath = kwargs.get('saspath', None)
         if insaspath:
-           if lock and len(path):
+           if lock and path is not None:
               print("Parameter 'saspath' passed to SAS_session was ignored due to configuration restriction.")
            else:
               path = insaspath
 
         injava = kwargs.get('java', None)
         if injava:
-           if lock and len(java):
+           if lock and java is not None:
               print("Parameter 'java' passed to SAS_session was ignored due to configuration restriction.")
            else:
               java = injava
 
         inprov = kwargs.get('provider', None)
         if inprov:
-           if lock and len(provider):
+           if lock and provider is not None:
               print("Parameter 'provider' passed to SAS_session was ignored due to configuration restriction.")
            else:
               provider = inprov
 
         if java is not None:
             self.mode = 'IOM'
+        elif url is not None:
+            self.mode = 'HTTP'
         elif ip is not None:
             self.mode = 'HTTP'
         elif ssh is not None:
@@ -1129,8 +1139,9 @@ class SASsession():
      
     def df2sd(self, df: 'pandas.DataFrame', table: str = '_df', libref: str = '',
               results: str = '', keep_outer_quotes: bool = False,
-                                 embedded_newlines: bool = False, 
-              LF: str = '\x01', CR: str = '\x02', colsep: str = '\x03',
+                                 embedded_newlines: bool = True, 
+              LF: str = '\x01', CR: str = '\x02',
+              colsep: str = '\x03', colrep: str = ' ',
               datetimes: dict={}, outfmts: dict={}, labels: dict={}) -> 'SASdata':
         """
         This is an alias for 'dataframe2sasdata'. Why type all that?
@@ -1139,22 +1150,30 @@ class SASsession():
         :param table: the name of the SAS Data Set to create
         :param libref: the libref for the SAS Data Set being created. Defaults to WORK, or USER if assigned
         :param results: format of results, SASsession.results is default, PANDAS, HTML or TEXT are the alternatives
+
+        As of version 3.5.0, keep_outer_quotes is deprecated and embedded_newlines defaults to True
+
         :param keep_outer_quotes: the defualt is for SAS to strip outer quotes from delimitted data. This lets you keep them
         :param embedded_newlines: if any char columns have embedded CR or LF, set this to True to get them iported into the SAS data set
+
+        colrep is new as of version 3.5.0
+
         :param LF: if embedded_newlines=True, the chacter to use for LF when transferring the data; defaults to hex(1)
         :param CR: if embedded_newlines=True, the chacter to use for CR when transferring the data; defaults to hex(2)
         :param colsep: the column seperator character used for streaming the delimmited data to SAS defaults to hex(3)
+        :param colrep: the char to convert to for any embedded colsep, LF, CR chars in the data; defaults to  ' '
         :param datetimes: dict with column names as keys and values of 'date' or 'time' to create SAS date or times instead of datetimes
         :param outfmts: dict with column names and SAS formats to assign to the new SAS data set
         :return: SASdata object
         """
         return self.dataframe2sasdata(df, table, libref, results, keep_outer_quotes, embedded_newlines, 
-                                      LF, CR, colsep, datetimes, outfmts, labels)
+                                      LF, CR, colsep, colrep, datetimes, outfmts, labels)
 
     def dataframe2sasdata(self, df: 'pandas.DataFrame', table: str = '_df', libref: str = '', 
                           results: str = '', keep_outer_quotes: bool = False,
-                                             embedded_newlines: bool = False, 
-                          LF: str = '\x01', CR: str = '\x02', colsep: str = '\x03',
+                                             embedded_newlines: bool = True, 
+                          LF: str = '\x01', CR: str = '\x02',
+                          colsep: str = '\x03', colrep: str = ' ',
                           datetimes: dict={}, outfmts: dict={}, labels: dict={}) -> 'SASdata':
         """
         This method imports a Pandas Data Frame to a SAS Data Set, returning the SASdata object for the new Data Set.
@@ -1163,11 +1182,18 @@ class SASsession():
         :param table: the name of the SAS Data Set to create
         :param libref: the libref for the SAS Data Set being created. Defaults to WORK, or USER if assigned
         :param results: format of results, SASsession.results is default, PANDAS, HTML or TEXT are the alternatives
+
+        As of version 3.5.0, keep_outer_quotes is deprecated and embedded_newlines defaults to True
+
         :param keep_outer_quotes: the defualt is for SAS to strip outer quotes from delimitted data. This lets you keep them
         :param embedded_newlines: if any char columns have embedded CR or LF, set this to True to get them iported into the SAS data set
+
+        colrep is new as of version 3.5.0
+
         :param LF: if embedded_newlines=True, the chacter to use for LF when transferring the data; defaults to hex(1) 
         :param CR: if embedded_newlines=True, the chacter to use for CR when transferring the data; defaults to hex(2) 
         :param colsep: the column seperator character used for streaming the delimmited data to SAS defaults to hex(3) 
+        :param colrep: the char to convert to for any embedded colsep, LF, CR chars in the data; defaults to  ' '
         :param datetimes: dict with column names as keys and values of 'date' or 'time' to create SAS date or times instead of datetimes
         :param outfmts: dict with column names and SAS formats to assign to the new SAS data set
         :return: SASdata object
@@ -1187,7 +1213,7 @@ class SASsession():
             return None
         else:
             self._io.dataframe2sasdata(df, table, libref, keep_outer_quotes, embedded_newlines, 
-                                       LF, CR, colsep, datetimes, outfmts, labels)
+                                       LF, CR, colsep, colrep, datetimes, outfmts, labels)
 
         if self.exist(table, libref):
             return SASdata(self, libref, table, results)
@@ -1227,11 +1253,23 @@ class SASsession():
                     this has better support than CSV for embedded delimiters (commas), nulls, CR/LF that CSV \
                     has problems with 
 
+
+        For the CSV and DISK methods, the following 2 parameters are also available
+
+        :param tempfile: [optional] an OS path for a file to use for the local file; default it a temporary file that's cleaned up
+        :param tempkeep: if you specify your own file to use with tempfile=, this controls whether it's cleaned up after using it
+
+        For the MEMORY and DISK methods, the following 4 parameters are also available, depending upon access method
+
+        :param rowsep: the row seperator character to use; defaults to hex(1)
+        :param colsep: the column seperator character to use; defaults to hex(2)
+        :param rowrep: the char to convert to for any embedded rowsep chars, defaults to  ' '
+        :param colrep: the char to convert to for any embedded colsep chars, defaults to  ' '
+
+
         :param kwargs: a dictionary. These vary per access method, and are generally NOT needed.
                        They are either access method specific parms or specific pandas parms.
                        See the specific sasdata2dataframe* method in the access method for valid possibilities.
-                       These are generally here for diagnostics when researching issue, to override things or try
-                       different options.  
 
         :return: Pandas data frame
         """
@@ -1279,8 +1317,6 @@ class SASsession():
         :param kwargs: a dictionary. These vary per access method, and are generally NOT needed.
                        They are either access method specific parms or specific pandas parms.
                        See the specific sasdata2dataframe* method in the access method for valid possibilities.
-                       These are generally here for diagnostics when researching issue, to override things or try
-                       different options.  
 
         :return: Pandas data frame
         """
@@ -1290,7 +1326,8 @@ class SASsession():
                                       opts=opts, **kwargs)
 
     def sd2df_DISK(self, table: str, libref: str = '', dsopts: dict = None, tempfile: str = None, 
-                  tempkeep: bool = False, **kwargs) -> 'pandas.DataFrame':
+                  tempkeep: bool = False, rowsep: str = '\x01', colsep: str = '\x02',
+                  rowrep: str = ' ', colrep: str = ' ', **kwargs) -> 'pandas.DataFrame':
         """
         This is an alias for 'sasdata2dataframe' specifying method='DISK'. Why type all that?
 
@@ -1314,19 +1351,25 @@ class SASsession():
                               'firstobs' : '12'
                               'format'  : {'money': 'dollar10', 'time': 'tod5.'}
                              }
+
         :param tempfile: [optional] an OS path for a file to use for the local file; default it a temporary file that's cleaned up
         :param tempkeep: if you specify your own file to use with tempfile=, this controls whether it's cleaned up after using it
+
+        :param rowsep: the row seperator character to use; defaults to hex(1)
+        :param colsep: the column seperator character to use; defaults to hex(2)
+        :param rowrep: the char to convert to for any embedded rowsep chars, defaults to  ' '
+        :param colrep: the char to convert to for any embedded colsep chars, defaults to  ' '
+
+
         :param kwargs: a dictionary. These vary per access method, and are generally NOT needed.
                        They are either access method specific parms or specific pandas parms.
                        See the specific sasdata2dataframe* method in the access method for valid possibilities.
-                       These are generally here for diagnostics when researching issue, to override things or try
-                       different options.  
 
         :return: Pandas data frame
         """
         dsopts = dsopts if dsopts is not None else {}
         return self.sasdata2dataframe(table, libref, dsopts, method='DISK', tempfile=tempfile, tempkeep=tempkeep,
-                                      **kwargs)
+                                      rowsep=rowsep, colsep=colsep, rowrep=rowrep, colrep=colrep, **kwargs)
 
     def sasdata2dataframe(self, table: str, libref: str = '', dsopts: dict = None, 
                           method: str = 'MEMORY', **kwargs) -> 'pandas.DataFrame':
@@ -1358,15 +1401,27 @@ class SASsession():
 
            - MEMORY the original method. Streams the data over and builds the dataframe on the fly in memory
            - CSV    uses an intermediary Proc Export csv file and pandas read_csv() to import it; faster for large data
-           - DISK   uses the original (MEMORY) method, but persists to disk and uses pandas read to import.
-                    this has better support than CSV for embedded delimiters (commas), nulls, CR/LF that CSV
+           - DISK   uses the original (MEMORY) method, but persists to disk and uses pandas read to import.  \
+                    this has better support than CSV for embedded delimiters (commas), nulls, CR/LF that CSV \
                     has problems with 
 
+
+        For the CSV and DISK methods, the following 2 parameters are also available
+
+        :param tempfile: [optional] an OS path for a file to use for the local file; default it a temporary file that's cleaned up
+        :param tempkeep: if you specify your own file to use with tempfile=, this controls whether it's cleaned up after using it
+
+        For the MEMORY and DISK methods, the following 4 parameters are also available, depending upon access method
+
+        :param rowsep: the row seperator character to use; defaults to hex(1)
+        :param colsep: the column seperator character to use; defaults to hex(2)
+        :param rowrep: the char to convert to for any embedded rowsep chars, defaults to  ' '
+        :param colrep: the char to convert to for any embedded colsep chars, defaults to  ' '
+
+
         :param kwargs: a dictionary. These vary per access method, and are generally NOT needed.
                        They are either access method specific parms or specific pandas parms.
                        See the specific sasdata2dataframe* method in the access method for valid possibilities.
-                       These are generally here for diagnostics when researching issue, to override things or try
-                       different options.  
 
         :return: Pandas data frame
         """
