@@ -44,6 +44,7 @@ class SASconfigHTTP:
       self.name      = session._sb.sascfg.name
       cfg            = getattr(SAScfg, self.name)
 
+      self.url       = cfg.get('url', '')
       self.ip        = cfg.get('ip', '')
       self.port      = cfg.get('port', None)
       self.ctxname   = cfg.get('context', '')
@@ -81,6 +82,13 @@ class SASconfigHTTP:
       self.verbose = self.cfgopts.get('verbose', True)
       self.verbose = kwargs.get('verbose', self.verbose)
 
+      inurl = kwargs.get('url', '')             
+      if len(inurl) > 0:
+         if lock and len(self.url):
+            print("Parameter 'url' passed to SAS_session was ignored due to configuration restriction.")
+         else:
+            self.url = inurl   
+
       inip = kwargs.get('ip', '')             
       if len(inip) > 0:
          if lock and len(self.ip):
@@ -169,6 +177,16 @@ class SASconfigHTTP:
          else:
             self.authkey = inak   
 
+      if len(self.url) > 0:
+         http = self.url.split('://')
+         hp   = http[1].split(':')
+         if http[0].lower() in ['http', 'https']:
+            self.ip   = hp[0]
+            self.port = hp[1] if len(hp) > 1 else self.port
+            self.ssl  = True if 's' in http[0].lower() else False
+         else:
+            print("Parameter 'url' not in recognized format. Expeting 'http[s]://host[:port]'. Ignoring parameter.")
+
       while len(self.ip) == 0:
          if not lock:
             self.ip = self._prompt("Please enter the host (ip address) you are trying to connect to: ")
@@ -365,8 +383,8 @@ class SASconfigHTTP:
       conn.close()
 
       if status > 299:
-         print("Failure in GET Contexts. Status="+str(status)+"\nResponse="+resp.decode(self.encoding))
-         return None
+         fmsg = "Failure in GET Contexts. Status="+str(status)+"\nResponse="+resp.decode(self.encoding)
+         raise SASHTTPconnectionError(msg=fmsg)
 
       js = json.loads(resp.decode(self.encoding))
       contexts = js.get('items')
@@ -796,7 +814,7 @@ class SASsessionHTTP():
                conn.request('GET', uri, headers=headers)
                req = conn.getresponse()
                resp = req.read()
-               if resp not in [b'running', 'pending']:
+               if resp not in [b'running', b'pending']:
                   done = True
                   break
                sleep(.5)
@@ -1109,8 +1127,9 @@ class SASsessionHTTP():
 
    def dataframe2sasdata(self, df: '<Pandas Data Frame object>', table: str ='a', 
                          libref: str ="", keep_outer_quotes: bool=False,
-                                          embedded_newlines: bool=False,
-                         LF: str = '\x01', CR: str = '\x02', colsep: str = '\x03',
+                                          embedded_newlines: bool=True,
+                         LF: str = '\x01', CR: str = '\x02',
+                         colsep: str = '\x03', colrep: str = ' ',
                          datetimes: dict={}, outfmts: dict={}, labels: dict={}):
       '''
       This method imports a Pandas Data Frame to a SAS Data Set, returning the SASdata object for the new Data Set.
@@ -1212,7 +1231,7 @@ class SASsessionHTTP():
       if len(format):
          code += "format "+format+";\n"
       code += label
-      code += "infile datalines delimiter="+delim+" DSD STOPOVER;\ninput @;\nif _infile_ = '' then delete;\ninput "+input+";\n"+xlate+";\ndatalines4;"
+      code += "infile datalines delimiter="+delim+" STOPOVER;\ninput @;\nif _infile_ = '' then delete;\ninput "+input+";\n"+xlate+";\ndatalines4;"
       self._asubmit(code, "text")
 
       code = ""
@@ -1227,7 +1246,9 @@ class SASsessionHTTP():
                if var == 'nan':
                   var = ' '
                else:
+                  var = var.replace(colsep, colrep)
                   if embedded_newlines:
+                     var = var.replace(LF, colrep).replace(CR, colrep)
                      var = var.replace('\n', LF).replace('\r', CR)
             elif dts[col] == 'B':
                var = str(int(row[col]))
@@ -1249,19 +1270,29 @@ class SASsessionHTTP():
       ll = self.submit("run;", 'text')
       return
 
-   def sasdata2dataframe(self, table: str, libref: str ='', dsopts: dict ={}, **kwargs) -> '<Pandas Data Frame object>':
+   def sasdata2dataframe(self, table: str, libref: str ='', dsopts: dict = None,
+                         rowsep: str = '\x01', colsep: str = '\x02',
+                         rowrep: str = ' ',    colrep: str = ' ',
+                         **kwargs) -> '<Pandas Data Frame object>':
       '''
       This method exports the SAS Data Set to a Pandas Data Frame, returning the Data Frame object.
       table   - the name of the SAS Data Set you want to export to a Pandas Data Frame
       libref  - the libref for the SAS Data Set.
       dsopts  - data set options for the input SAS Data Set
+      Only for DISK version:
+      rowsep  - the row seperator character to use; defaults to '\x01'
+      colsep  - the column seperator character to use; defaults to '\x02'
+      rowrep  - the char to convert to for any embedded rowsep chars, defaults to  ' '
+      colrep  - the char to convert to for any embedded colsep chars, defaults to  ' '
       '''
+      dsopts = dsopts if dsopts is not None else {}
 
       method = kwargs.pop('method', None)
       if   method and method.lower() == 'csv':
          return self.sasdata2dataframeCSV(table, libref, dsopts, **kwargs)
       elif method and method.lower() == 'disk':
-         return self.sasdata2dataframeDISK(table, libref, dsopts, **kwargs)
+         return self.sasdata2dataframeDISK(table, libref, dsopts, rowsep, colsep,
+                                           rowrep, colrep, **kwargs)
 
       my_fmts = kwargs.pop('my_fmts', False)
       k_dts   = kwargs.pop('dtype',   None)
@@ -1346,7 +1377,7 @@ class SASsessionHTTP():
       uri = "/compute/sessions/"+self.pid+"/data/work/saspy_ds2df/rows"
 
       r     = []
-      df    = None
+      df    = pd.DataFrame.from_records(r, columns=varlist)
       trows = kwargs.get('trows', None)
       if not trows:
          trows = 100000
@@ -1419,7 +1450,8 @@ class SASsessionHTTP():
       return df
 
 
-   def sasdata2dataframeCSV(self, table: str, libref: str ='', dsopts: dict ={}, tempfile: str=None, tempkeep: bool=False, **kwargs) -> '<Pandas Data Frame object>':
+   def sasdata2dataframeCSV(self, table: str, libref: str ='', dsopts: dict =None, opts: dict = None, 
+                            tempfile: str=None, tempkeep: bool=False, **kwargs) -> '<Pandas Data Frame object>':
       '''
       This method exports the SAS Data Set to a Pandas Data Frame, returning the Data Frame object.
       table    - the name of the SAS Data Set you want to export to a Pandas Data Frame
@@ -1435,7 +1467,8 @@ class SASsessionHTTP():
       dtype   - this is the parameter to Pandas read_csv, overriding what saspy generates and uses
       my_fmts - bool: if True, overrides the formats saspy would use, using those on the data set or in dsopts=
       '''
-      opts   = kwargs.pop('opts', {})
+      dsopts = dsopts if dsopts is not None else {}
+      opts   = opts   if   opts is not None else {}
 
       if libref:
          tabname = libref+".'"+table.strip()+"'n "
@@ -1558,8 +1591,9 @@ class SASsessionHTTP():
 
       return df
 
-   def sasdata2dataframeDISK(self, table: str, libref: str ='', dsopts: dict ={},
-                             rowsep: str = '\x01', colsep: str = '\x02', tempfile: str=None, 
+   def sasdata2dataframeDISK(self, table: str, libref: str ='', dsopts: dict = None,
+                             rowsep: str = '\x01', colsep: str = '\x02',
+                             rowrep: str = ' ',    colrep: str = ' ', tempfile: str=None, 
                              tempkeep: bool=False, **kwargs) -> '<Pandas Data Frame object>':
       '''
       This method exports the SAS Data Set to a Pandas Data Frame, returning the Data Frame object.
@@ -1568,6 +1602,8 @@ class SASsessionHTTP():
       dsopts   - data set options for the input SAS Data Set
       rowsep   - the row seperator character to use; defaults to '\x01'
       colsep   - the column seperator character to use; defaults to '\x02'
+      rowrep  - the char to convert to for any embedded rowsep chars, defaults to  ' '
+      colrep  - the char to convert to for any embedded colsep chars, defaults to  ' '
       tempfile - file to use to store CSV, else temporary file will be used.
       tempkeep - if you specify your own file to use with tempfile=, this controls whether it's cleaned up after using it
 
@@ -1577,7 +1613,7 @@ class SASsessionHTTP():
       dtype   - this is the parameter to Pandas read_csv, overriding what saspy generates and uses
       my_fmts - bool: if True, overrides the formats saspy would use, using those on the data set or in dsopts=
       '''
-      opts   = kwargs.pop('opts', {})
+      dsopts = dsopts if dsopts is not None else {}
 
       if libref:
          tabname = libref+".'"+table.strip()+"'n "
@@ -1667,8 +1703,20 @@ class SASsessionHTTP():
                if i % 10 == 0:
                   code +='\n'
 
-      code += "file _tomodsx lrecl=1 recfm=f encoding=binary;\n"
-
+      code += "\nfile _tomodsx lrecl=1 recfm=f encoding=binary;\n"
+      for i in range(nvars):
+         if vartype[i] != 'FLOAT':
+            code += "'"+varlist[i]+"'n = translate('"
+            code +=     varlist[i]+"'n, '{}'x, '{}'x); ".format(   \
+                        '%02x%02x' %                               \
+                        (ord(rowrep.encode(self.sascfg.encoding)), \
+                         ord(colrep.encode(self.sascfg.encoding))),
+                        '%02x%02x' %                               \
+                        (ord(rowsep.encode(self.sascfg.encoding)), \
+                         ord(colsep.encode(self.sascfg.encoding))))
+            if i % 10 == 0:
+               code +='\n'
+      code += "\n"
       last  = len(varlist)-1
       for i in range(nvars):
          code += "put '"+varlist[i]+"'n "
@@ -1699,9 +1747,11 @@ class SASsessionHTTP():
 
       miss = ['.', ' ']
 
+      quoting = kwargs.pop('quoting', 3)
+
       df = pd.read_csv(tmpcsv, index_col=False, engine='c', header=None, names=varlist, 
                        sep=colsep, lineterminator=rowsep, dtype=dts, na_values=miss,
-                       encoding=self.sascfg.encoding, **kwargs)
+                       encoding=self.sascfg.encoding, quoting=quoting, **kwargs)
 
       if tmpdir:
          tmpdir.cleanup()
@@ -1716,15 +1766,3 @@ class SASsessionHTTP():
                   df[varlist[i]] = pd.to_datetime(df[varlist[i]], errors='coerce')
 
       return df
-
-
-if __name__ == "__main__":
-    startsas()
-
-    submit(sys.argv[1], "text")
-
-    print(_getlog())
-    print(_getlsttxt())
-
-    endsas()
-
