@@ -1430,8 +1430,9 @@ Will use HTML5 for this SASsession.""")
 
    def dataframe2sasdata(self, df: '<Pandas Data Frame object>', table: str ='a', 
                          libref: str ="", keep_outer_quotes: bool=False,
-                                          embedded_newlines: bool=False,
-                         LF: str = '\x01', CR: str = '\x02', colsep: str = '\x03',
+                                          embedded_newlines: bool=True,
+                         LF: str = '\x01', CR: str = '\x02',
+                         colsep: str = '\x03', colrep: str = ' ',
                          datetimes: dict={}, outfmts: dict={}, labels: dict={}):
       """
       This method imports a Pandas Data Frame to a SAS Data Set, returning the SASdata object for the new Data Set.
@@ -1533,7 +1534,7 @@ Will use HTML5 for this SASsession.""")
       if len(format):
          code += "format "+format+";\n"
       code += label
-      code += "infile datalines delimiter="+delim+" DSD STOPOVER;\ninput @;\nif _infile_ = '' then delete;\ninput "+input+";\n"+xlate+";\ndatalines4;"
+      code += "infile datalines delimiter="+delim+" STOPOVER;\ninput @;\nif _infile_ = '' then delete;\ninput "+input+";\n"+xlate+";\ndatalines4;"
       self._asubmit(code, "text")
 
       code = ""
@@ -1548,7 +1549,9 @@ Will use HTML5 for this SASsession.""")
                if var == 'nan':
                   var = ' '
                else:
+                  var = var.replace(colsep, colrep)
                   if embedded_newlines:
+                     var = var.replace(LF, colrep).replace(CR, colrep)
                      var = var.replace('\n', LF).replace('\r', CR)
             elif dts[col] == 'B':
                var = str(int(row[col]))
@@ -1570,14 +1573,18 @@ Will use HTML5 for this SASsession.""")
       ll = self.submit("run;", 'text')
       return
 
-   def sasdata2dataframe(self, table: str, libref: str ='', dsopts: dict = None, rowsep: str = '\x01',
-                         colsep: str = '\x02', **kwargs) -> '<Pandas Data Frame object>':
+   def sasdata2dataframe(self, table: str, libref: str ='', dsopts: dict = None,
+                         rowsep: str = '\x01', colsep: str = '\x02',
+                         rowrep: str = ' ',    colrep: str = ' ',
+                         **kwargs) -> '<Pandas Data Frame object>':
       """
       This method exports the SAS Data Set to a Pandas Data Frame, returning the Data Frame object.
       table   - the name of the SAS Data Set you want to export to a Pandas Data Frame
       libref  - the libref for the SAS Data Set.
       rowsep  - the row seperator character to use; defaults to '\x01'
       colsep  - the column seperator character to use; defaults to '\x02'
+      rowrep  - the char to convert to for any embedded rowsep chars, defaults to  ' '
+      colrep  - the char to convert to for any embedded colsep chars, defaults to  ' '
       """
       dsopts = dsopts if dsopts is not None else {}
 
@@ -1585,7 +1592,8 @@ Will use HTML5 for this SASsession.""")
       if   method and method.lower() == 'csv':
          return self.sasdata2dataframeCSV(table, libref, dsopts, **kwargs)
       elif method and method.lower() == 'disk':
-         return self.sasdata2dataframeDISK(table, libref, dsopts, **kwargs)
+         return self.sasdata2dataframeDISK(table, libref, dsopts, rowsep, colsep,
+                                           rowrep, colrep, **kwargs)
 
       my_fmts = kwargs.pop('my_fmts', False)
       k_dts   = kwargs.pop('dtype',   None)
@@ -1677,22 +1685,48 @@ Will use HTML5 for this SASsession.""")
 
       if self._sb.m5dsbug:
          rsep = colsep+rowsep+'\n'
-         code += "file "+self._tomods1.decode()+" dlm="+cdelim+" termstr=NL;\nput "
+         code += "\nfile "+self._tomods1.decode()+" dlm="+cdelim+" termstr=NL;\n"
+         for i in range(nvars):
+            if vartype[i] != 'N':
+               code += "'"+varlist[i]+"'n = translate('"
+               code +=     varlist[i]+"'n, '{}'x, '{}'x); ".format(   \
+                           '%02x%02x' %                               \
+                           (ord(rowrep.encode(self.sascfg.encoding)), \
+                            ord(colrep.encode(self.sascfg.encoding))),
+                           '%02x%02x' %                               \
+                           (ord(rowsep.encode(self.sascfg.encoding)), \
+                            ord(colsep.encode(self.sascfg.encoding))))
+               if i % 10 == 0:
+                  code +='\n'
+         code += "\nput "
          for i in range(nvars):
             code += " '"+varlist[i]+"'n "
             if i % 10 == 0:
                code +='\n'
          code += rdelim+";\nrun;"
       else:
-         rsep = rowsep
-         code += "file "+self._tomods1.decode()+" lrecl=1 recfm=f encoding=binary;\n"
+         rsep = rowsep+rowsep
+         code += "\nfile "+self._tomods1.decode()+" lrecl=1 recfm=f encoding=binary;\n"
+         for i in range(nvars):
+            if vartype[i] != 'N':
+               code += "'"+varlist[i]+"'n = translate('"
+               code +=     varlist[i]+"'n, '{}'x, '{}'x); ".format(   \
+                           '%02x%02x' %                               \
+                           (ord(rowrep.encode(self.sascfg.encoding)), \
+                            ord(colrep.encode(self.sascfg.encoding))),
+                           '%02x%02x' %                               \
+                           (ord(rowsep.encode(self.sascfg.encoding)), \
+                            ord(colsep.encode(self.sascfg.encoding))))
+               if i % 10 == 0:
+                  code +='\n'
+         code += "\n"
          last  = len(varlist)-1
          for i in range(nvars):
             code += "put '"+varlist[i]+"'n "
             if i != last:
                code += cdelim+'; '
             else:
-               code += rdelim+'; '
+               code += rdelim+rdelim+'; '
             if i % 10 == 0:
                code +='\n'
          code += "run;"
@@ -1786,6 +1820,7 @@ Will use HTML5 for this SASsession.""")
                    if logf.count(logcodeo) >= 1:
                       bail = True
          done = True
+         self._log += logf
 
       if len(r) > 0 or df is None:
          tdf = pd.DataFrame.from_records(r, columns=varlist)
@@ -1808,7 +1843,8 @@ Will use HTML5 for this SASsession.""")
 
       return df
 
-   def sasdata2dataframeCSV(self, table: str, libref: str ='', dsopts: dict = None, tempfile: str=None, tempkeep: bool=False, **kwargs) -> '<Pandas Data Frame object>':
+   def sasdata2dataframeCSV(self, table: str, libref: str ='', dsopts: dict = None, opts: dict = None, 
+                            tempfile: str=None, tempkeep: bool=False, **kwargs) -> '<Pandas Data Frame object>':
       """
       This method exports the SAS Data Set to a Pandas Data Frame, returning the Data Frame object.
       table    - the name of the SAS Data Set you want to export to a Pandas Data Frame
@@ -1825,7 +1861,7 @@ Will use HTML5 for this SASsession.""")
       my_fmts - bool: if True, overrides the formats saspy would use, using those on the data set or in dsopts=
       """
       dsopts = dsopts if dsopts is not None else {}
-      opts   = kwargs.pop('opts', {})
+      opts   = opts   if   opts is not None else {}
 
       logf     = ''
       lstf     = ''
@@ -2060,7 +2096,8 @@ Will use HTML5 for this SASsession.""")
       return df
 
    def sasdata2dataframeDISK(self, table: str, libref: str ='', dsopts: dict = None,
-                             rowsep: str = '\x01', colsep: str = '\x02', tempfile: str=None, 
+                             rowsep: str = '\x01', colsep: str = '\x02',
+                             rowrep: str = ' ',    colrep: str = ' ', tempfile: str=None, 
                              tempkeep: bool=False, **kwargs) -> '<Pandas Data Frame object>':
       """
       This method exports the SAS Data Set to a Pandas Data Frame, returning the Data Frame object.
@@ -2069,6 +2106,8 @@ Will use HTML5 for this SASsession.""")
       dsopts   - data set options for the input SAS Data Set
       rowsep   - the row seperator character to use; defaults to '\x01'
       colsep   - the column seperator character to use; defaults to '\x02'
+      rowrep  - the char to convert to for any embedded rowsep chars, defaults to  ' '
+      colrep  - the char to convert to for any embedded colsep chars, defaults to  ' '
       tempfile - file to use to store CSV, else temporary file will be used.
       tempkeep - if you specify your own file to use with tempfile=, this controls whether it's cleaned up after using it
 
@@ -2079,7 +2118,6 @@ Will use HTML5 for this SASsession.""")
       my_fmts - bool: if True, overrides the formats saspy would use, using those on the data set or in dsopts=
       """
       dsopts = dsopts if dsopts is not None else {}
-      opts   = kwargs.pop('opts', {})
 
       logf     = ''
       lstf     = ''
@@ -2194,7 +2232,20 @@ Will use HTML5 for this SASsession.""")
 
       if self._sb.m5dsbug:
          rsep = colsep+rowsep+'\n'
-         code += "file "+outname+" dlm="+cdelim+" termstr=NL;\nput "
+         code += "\nfile "+outname+" dlm="+cdelim+" termstr=NL;\n"
+         for i in range(nvars):
+            if vartype[i] != 'N':
+               code += "'"+varlist[i]+"'n = translate('"
+               code +=     varlist[i]+"'n, '{}'x, '{}'x); ".format(   \
+                           '%02x%02x' %                               \
+                           (ord(rowrep.encode(self.sascfg.encoding)), \
+                            ord(colrep.encode(self.sascfg.encoding))),
+                           '%02x%02x' %                               \
+                           (ord(rowsep.encode(self.sascfg.encoding)), \
+                            ord(colsep.encode(self.sascfg.encoding))))
+               if i % 10 == 0:
+                  code +='\n'
+         code += "\nput "
          for i in range(nvars):
             code += " '"+varlist[i]+"'n "
             if i % 10 == 0:
@@ -2202,7 +2253,20 @@ Will use HTML5 for this SASsession.""")
          code += rdelim+";\nrun;"
       else:
          rsep = rowsep
-         code += "file "+outname+" lrecl=1 recfm=f encoding=binary;\n"
+         code += "\nfile "+outname+" lrecl=1 recfm=f encoding=binary;\n"
+         for i in range(nvars):
+            if vartype[i] != 'N':
+               code += "'"+varlist[i]+"'n = translate('"
+               code +=     varlist[i]+"'n, '{}'x, '{}'x); ".format(   \
+                           '%02x%02x' %                               \
+                           (ord(rowrep.encode(self.sascfg.encoding)), \
+                            ord(colrep.encode(self.sascfg.encoding))),
+                           '%02x%02x' %                               \
+                           (ord(rowsep.encode(self.sascfg.encoding)), \
+                            ord(colsep.encode(self.sascfg.encoding))))
+               if i % 10 == 0:
+                  code +='\n'
+         code += "\n"
          last  = len(varlist)-1
          for i in range(nvars):
             code += "put '"+varlist[i]+"'n "
@@ -2326,9 +2390,11 @@ Will use HTML5 for this SASsession.""")
 
       miss = ['.', ' ']
 
+      quoting = kwargs.pop('quoting', 3)
+
       df = pd.read_csv(tmpcsv, index_col=False, engine='c', header=None, names=varlist, 
                        sep=colsep, lineterminator=rowsep, dtype=dts, na_values=miss,
-                       encoding=enc, **kwargs)
+                       encoding=enc, quoting=quoting, **kwargs)
 
       if tmpdir:
          tmpdir.cleanup()
@@ -2343,13 +2409,3 @@ Will use HTML5 for this SASsession.""")
                   df[varlist[i]] = pd.to_datetime(df[varlist[i]], errors='coerce')
 
       return df
-
-if __name__ == "__main__":
-    startsas()
-
-    submit(sys.argv[1], "text")
-
-    print(_getlog())
-    print(_getlsttxt())
-
-    endsas()
