@@ -1407,8 +1407,9 @@ Will use HTML5 for this SASsession.""")
 
    def dataframe2sasdata(self, df: '<Pandas Data Frame object>', table: str ='a',
                          libref: str ="", keep_outer_quotes: bool=False,
-                                          embedded_newlines: bool=False,
-                         LF: str = '\x01', CR: str = '\x02', colsep: str = '\x03',
+                                          embedded_newlines: bool=True,
+                         LF: str = '\x01', CR: str = '\x02',
+                         colsep: str = '\x03', colrep: str = ' ',
                          datetimes: dict={}, outfmts: dict={}, labels: dict={}):
       """
       This method imports a Pandas Data Frame to a SAS Data Set, returning the SASdata object for the new Data Set.
@@ -1510,7 +1511,8 @@ Will use HTML5 for this SASsession.""")
       if len(format):
          code += "format "+format+";\n"
       code += label
-      code += "infile datalines delimiter="+delim+" DSD STOPOVER;\n input "+input+";\n"+xlate+";\n datalines4;"
+      #code += "infile datalines delimiter="+delim+" STOPOVER;\n input "+input+";\n"+xlate+";\n datalines4;"
+      code += "infile datalines delimiter="+delim+" STOPOVER;\ninput @;\nif _infile_ = '' then delete;\ninput "+input+";\n"+xlate+";\ndatalines4;"
       self._asubmit(code, "text")
 
       for row in df.itertuples(index=False):
@@ -1525,7 +1527,9 @@ Will use HTML5 for this SASsession.""")
                if var == 'nan':
                   var = ' '
                else:
+                  var = var.replace(colsep, colrep)
                   if embedded_newlines:
+                     var = var.replace(LF, colrep).replace(CR, colrep)
                      var = var.replace('\n', LF).replace('\r', CR)
             elif dts[col] == 'B':
                var = str(int(row[col]))
@@ -1548,14 +1552,18 @@ Will use HTML5 for this SASsession.""")
       ll = self.submit("run;", 'text')
       return
 
-   def sasdata2dataframe(self, table: str, libref: str ='', dsopts: dict = None, rowsep: str = '\x01',
-                         colsep: str = '\x02', wait: int=10, **kwargs) -> '<Pandas Data Frame object>':
+   def sasdata2dataframe(self, table: str, libref: str ='', dsopts: dict = None,
+                         rowsep: str = '\x01', colsep: str = '\x02',
+                         rowrep: str = ' ',    colrep: str = ' ', 
+                         port: int=0, wait: int=10, **kwargs) -> '<Pandas Data Frame object>':
       """
       This method exports the SAS Data Set to a Pandas Data Frame, returning the Data Frame object.
       table   - the name of the SAS Data Set you want to export to a Pandas Data Frame
       libref  - the libref for the SAS Data Set.
       rowsep  - the row seperator character to use; defaults to '\x01'
       colsep  - the column seperator character to use; defaults to '\x02'
+      rowrep  - the char to convert to for any embedded rowsep chars, defaults to  ' '
+      colrep  - the char to convert to for any embedded colsep chars, defaults to  ' '
       port    - port to use for socket. Defaults to 0 which uses a random available ephemeral port
       wait    - seconds to wait for socket connection from SAS; catches hang if an error in SAS. 0 = no timeout
       """
@@ -1563,9 +1571,10 @@ Will use HTML5 for this SASsession.""")
 
       method = kwargs.pop('method', None)
       if   method and method.lower() == 'csv':
-         return self.sasdata2dataframeCSV(table, libref, dsopts, wait=wait, **kwargs)
+         return self.sasdata2dataframeCSV(table, libref, dsopts, port=port, wait=wait, **kwargs)
       elif method and method.lower() == 'disk':
-         return self.sasdata2dataframeDISK(table, libref, dsopts, rowsep, colsep, wait=wait, **kwargs)
+         return self.sasdata2dataframeDISK(table, libref, dsopts, rowsep, colsep,
+                                           rowrep, colrep, port=port, wait=wait, **kwargs)
 
       my_fmts = kwargs.pop('my_fmts', False)
       k_dts   = kwargs.pop('dtype',   None)
@@ -1575,8 +1584,6 @@ Will use HTML5 for this SASsession.""")
          if k_dts is not None:
             print("'dtype=' is only used with the CSV or DISK version of this method. option ignored.")
 
-      port =  kwargs.get('port', 0)
-
       if port==0 and self.sascfg.tunnel:
          # we are using a tunnel; default to that port
          port = self.sascfg.tunnel
@@ -1684,7 +1691,20 @@ Will use HTML5 for this SASsession.""")
       if self._sb.m5dsbug:
          rsep = colsep+rowsep+'\n'
          csep = colsep
-         code += "file sock dlm="+cdelim+";\nput "
+         code += "\nfile sock dlm="+cdelim+";\n"
+         for i in range(nvars):
+            if vartype[i] != 'N':
+               code += "'"+varlist[i]+"'n = translate('"
+               code +=     varlist[i]+"'n, '{}'x, '{}'x); ".format(   \
+                           '%02x%02x' %                               \
+                           (ord(rowrep.encode(self.sascfg.encoding)), \
+                            ord(colrep.encode(self.sascfg.encoding))),
+                           '%02x%02x' %                               \
+                           (ord(rowsep.encode(self.sascfg.encoding)), \
+                            ord(colsep.encode(self.sascfg.encoding))))
+               if i % 10 == 0:
+                  code +='\n'
+         code += "\nput "
          for i in range(nvars):
             code += " '"+varlist[i]+"'n "
             if i % 10 == 0:
@@ -1693,7 +1713,20 @@ Will use HTML5 for this SASsession.""")
       else:
          rsep = ' '+rowsep
          csep = ' '+colsep
-         code += "file sock; "
+         code += "\nfile sock;\n"
+         for i in range(nvars):
+            if vartype[i] != 'N':
+               code += "'"+varlist[i]+"'n = translate('"
+               code +=     varlist[i]+"'n, '{}'x, '{}'x); ".format(   \
+                           '%02x%02x' %                               \
+                           (ord(rowrep.encode(self.sascfg.encoding)), \
+                            ord(colrep.encode(self.sascfg.encoding))),
+                           '%02x%02x' %                               \
+                           (ord(rowsep.encode(self.sascfg.encoding)), \
+                            ord(colsep.encode(self.sascfg.encoding))))
+               if i % 10 == 0:
+                  code +='\n'
+         code += "\n"
          last  = len(varlist)-1
          for i in range(nvars):
             code += "put '"+varlist[i]+"'n "
@@ -1798,17 +1831,17 @@ Will use HTML5 for this SASsession.""")
 
       return df
 
-   def sasdata2dataframeCSV(self, table: str, libref: str ='', dsopts: dict = None, tempfile: str=None, 
-                            tempkeep: bool=False, wait: int=10, **kwargs) -> '<Pandas Data Frame object>':
+   def sasdata2dataframeCSV(self, table: str, libref: str ='', dsopts: dict = None, opts: dict = None, 
+                            tempfile: str=None, tempkeep: bool=False, port: int=0, wait: int=10, **kwargs) -> '<Pandas Data Frame object>':
       """
       This method exports the SAS Data Set to a Pandas Data Frame, returning the Data Frame object.
       table    - the name of the SAS Data Set you want to export to a Pandas Data Frame
       libref   - the libref for the SAS Data Set.
       dsopts   - data set options for the input SAS Data Set
       opts     - a dictionary containing any of the following Proc Export options(delimiter, putnames)
-      port     - port to use for socket. Defaults to 0 which uses a random available ephemeral port
       tempfile - file to use to store CSV, else temporary file will be used.
       tempkeep - if you specify your own file to use with tempfile=, this controls whether it's cleaned up after using it
+      port     - port to use for socket. Defaults to 0 which uses a random available ephemeral port
       wait     - seconds to wait for socket connection from SAS; catches hang if an error in SAS. 0 = no timeout
 
       These two options are for advanced usage. They override how saspy imports data. For more info
@@ -1818,8 +1851,7 @@ Will use HTML5 for this SASsession.""")
       my_fmts - bool: if True, overrides the formats saspy would use, using those on the data set or in dsopts=
       """
       dsopts = dsopts if dsopts is not None else {}
-      opts   = kwargs.pop('opts', {})
-      port   = kwargs.get('port', 0)
+      opts   = opts   if   opts is not None else {}
 
       if port==0 and self.sascfg.tunnel:
          # we are using a tunnel; default to that port
@@ -2015,8 +2047,9 @@ Will use HTML5 for this SASsession.""")
       return df
 
    def sasdata2dataframeDISK(self, table: str, libref: str ='', dsopts: dict = None,  
-                             rowsep: str = '\x01', colsep: str = '\x02', tempfile: str=None, 
-                             tempkeep: bool=False, wait: int=10, **kwargs) -> '<Pandas Data Frame object>':
+                             rowsep: str = '\x01', colsep: str = '\x02',
+                             rowrep: str = ' ',    colrep: str = ' ', tempfile: str=None, 
+                             tempkeep: bool=False, port: int=0, wait: int=10, **kwargs) -> '<Pandas Data Frame object>':
       """
       This method exports the SAS Data Set to a Pandas Data Frame, returning the Data Frame object.
       table    - the name of the SAS Data Set you want to export to a Pandas Data Frame
@@ -2024,9 +2057,11 @@ Will use HTML5 for this SASsession.""")
       dsopts   - data set options for the input SAS Data Set
       rowsep   - the row seperator character to use; defaults to '\x01'
       colsep   - the column seperator character to use; defaults to '\x02'
-      port     - port to use for socket. Defaults to 0 which uses a random available ephemeral port
+      rowrep  - the char to convert to for any embedded rowsep chars, defaults to  ' '
+      colrep  - the char to convert to for any embedded colsep chars, defaults to  ' '
       tempfile - file to use to store CSV, else temporary file will be used.
       tempkeep - if you specify your own file to use with tempfile=, this controls whether it's cleaned up after using it
+      port     - port to use for socket. Defaults to 0 which uses a random available ephemeral port
       wait     - seconds to wait for socket connection from SAS; catches hang if an error in SAS. 0 = no timeout
 
       These two options are for advanced usage. They override how saspy imports data. For more info
@@ -2036,8 +2071,6 @@ Will use HTML5 for this SASsession.""")
       my_fmts - bool: if True, overrides the formats saspy would use, using those on the data set or in dsopts=
       """
       dsopts = dsopts if dsopts is not None else {}
-      opts   = kwargs.pop('opts', {})
-      port   = kwargs.get('port', 0)
 
       if port==0 and self.sascfg.tunnel:
          # we are using a tunnel; default to that port
@@ -2162,7 +2195,20 @@ Will use HTML5 for this SASsession.""")
       if self._sb.m5dsbug:
          rsep = colsep+rowsep+'\n'
          csep = colsep
-         code += "file sock dlm="+cdelim+";\nput "
+         code += "\nfile sock dlm="+cdelim+";\n"
+         for i in range(nvars):
+            if vartype[i] != 'N':
+               code += "'"+varlist[i]+"'n = translate('"
+               code +=     varlist[i]+"'n, '{}'x, '{}'x); ".format(   \
+                           '%02x%02x' %                               \
+                           (ord(rowrep.encode(self.sascfg.encoding)), \
+                            ord(colrep.encode(self.sascfg.encoding))),
+                           '%02x%02x' %                               \
+                           (ord(rowsep.encode(self.sascfg.encoding)), \
+                            ord(colsep.encode(self.sascfg.encoding))))
+               if i % 10 == 0:
+                  code +='\n'
+         code += "\nput "
          for i in range(nvars):
             code += " '"+varlist[i]+"'n "
             if i % 10 == 0:
@@ -2171,7 +2217,20 @@ Will use HTML5 for this SASsession.""")
       else:
          rsep = ' '+rowsep
          csep = ' '+colsep
-         code += "file sock; "
+         code += "\nfile sock;\n"
+         for i in range(nvars):
+            if vartype[i] != 'N':
+               code += "'"+varlist[i]+"'n = translate('"
+               code +=     varlist[i]+"'n, '{}'x, '{}'x); ".format(   \
+                           '%02x%02x' %                               \
+                           (ord(rowrep.encode(self.sascfg.encoding)), \
+                            ord(colrep.encode(self.sascfg.encoding))),
+                           '%02x%02x' %                               \
+                           (ord(rowsep.encode(self.sascfg.encoding)), \
+                            ord(colsep.encode(self.sascfg.encoding))))
+               if i % 10 == 0:
+                  code +='\n'
+         code += "\n"
          last  = len(varlist)-1
          for i in range(nvars):
             code += "put '"+varlist[i]+"'n "
@@ -2252,12 +2311,14 @@ Will use HTML5 for this SASsession.""")
       else:
          dts = k_dts
 
-      miss = ['.', ' ']
+      miss = ['.', ' ', '. ']
+
+      quoting = kwargs.pop('quoting', 3)
 
       try:
          df = pd.read_csv(tmpcsv, index_col=False, engine='c', header=None, names=varlist, 
                           sep=colsep, lineterminator=rowsep, dtype=dts, na_values=miss,
-                          encoding=enc, **kwargs)
+                          encoding=enc, quoting=quoting, **kwargs)
       except FileNotFoundError:
          print("error occured in SAS during sasdata2dataframe. Trying to return the saslog instead of a data frame.")
          if tmpdir:
@@ -2277,13 +2338,3 @@ Will use HTML5 for this SASsession.""")
                   df[varlist[i]] = pd.to_datetime(df[varlist[i]], errors='coerce')
 
       return df
-
-if __name__ == "__main__":
-    startsas()
-
-    submit(sys.argv[1], "text")
-
-    print(_getlog())
-    print(_getlsttxt())
-
-    endsas()
